/*
 * Copyright (C) 2002 John David Ratliff
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;

import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;

/**
 * @author John David Ratliff
 * @version 1.0.0
 *
 * Class representing a Confetti like drawing window.
 */
public class Confetti {
    private DrawingWindow window;
    private ColorBar colorbar;

    /**
     * The default window width, if no width is specified.
     */
    public static final int DEFAULT_WIDTH = 400;

    /**
     * The default window height, if no height is specified.
     */
    public static final int DEFAULT_HEIGHT = 300;

    /**
     * The radius of the Confetti circles.
     */
    public static final int CIRCLE_RADIUS = 10;

    /**
     * Create a new Confetti drawing window.
     *
     * @param width The window's width.
     * @param height The window's height.
     * @param colors The number of colors on the Color Bar.
     */
    public Confetti(int width, int height, int colors) {
        this.window = new DrawingWindow("Confetti", Color.BLACK,
                                         width, height);

        // create and paint the ColorBar
        this.colorbar = new ColorBar(width, colors);
        this.colorbar.draw(this.window.getPen());
        
        // paint the Window
        this.window.repaint();

        // await user interaction
        this.waitForConfetti();
    }

    /**
     * Create a new Confetti Drawing Window.
     *
     * @param colors The number of colors on the Color Bar.
     */
    public Confetti(int colors) {
        this(DEFAULT_WIDTH, DEFAULT_HEIGHT, colors);
    }

    /**
     * Create a default Confetti Drawing Window.
     */
    public Confetti() {
        this(DEFAULT_WIDTH, DEFAULT_HEIGHT, ColorBar.DEFAULT_COLORS);
    }

    /**
     * Wait for the user to press the mouse buttons
     * and add Confetti where they press the mouse.
     */
    private void waitForConfetti() {
        Point p;
        Graphics2D pen = this.window.getPen();
        int newColor;
        int barWidth = window.width() / colorbar.getColors();
        int barHeight = colorbar.getHeight();

        // while we are still getting clicks from the Window
        while ((p = window.getMouseClick()) != null) {
            if (p.y <= barHeight) {
                // we selected a Color
                // find out which one
                newColor = p.x / barWidth;

                if (this.colorbar.getColor() != newColor) {
                    // only redraw when we have to
                    this.colorbar.setColor(newColor);
                    this.colorbar.draw(pen);
                }
            } else {
                // paint the Dot in the proper color
                pen.setPaint(colorbar.getPaletteColor(colorbar.getColor()));
                new Dot(p,CIRCLE_RADIUS).draw(pen);

                // if we obscured the Color Bar, repaint it
                if ((p.y - CIRCLE_RADIUS) < barHeight) {
                    this.colorbar.draw(pen);
                }
            }

            // finally, repaint the Window...
            this.window.repaint();
        }
    }

    /**
     * Start a new Confetti program.
     *
     * @param args not used...
     */
    public static void main(String[] args) {
        new Confetti();
    }
}

/**
 * @author John David Ratliff [jdratlif@indiana.edu]
 * @version 1.0.0
 *
 * Class representing a ColorBar.
 */
class ColorBar {
    private int width, height;
    private int colors, color = 0;

    /**
     * The default number of colors on the color bar.
     */
    public static final int DEFAULT_COLORS = 4;

    /**
     * The maximum possible colors on the color bar.
     */
    public static final int MAX_COLORS = 11;

    /**
     * The default colorbar height;
     */
    public static final int DEFAULT_HEIGHT = 35;

    /**
     * Create a new ColorBar of a certain width and
     * number of colors.
     *
     * @param width The width of the ColorBar.
     * @param height The height of the ColorBar.
     * @param colors The number of colors on the bar.
     */
    public ColorBar(int width, int height, int colors) {
        this.width = width;
        this.height = height;
        this.colors = colors;
    }
    
    /**
     * Create a new ColorBar of a certain width and
     * number of colors.
     *
     * @param width The width of the ColorBar.
     * @param colors The number of colors on the bar.
     */
    public ColorBar(int width, int colors) {
        this(width, DEFAULT_HEIGHT, colors);
    }

    /**
     * Get the bounding Rectangle2D object for a certain
     * color on the color bar.
     *
     * @param The position of the color on the Color Bar.
     * @return The Rectangle2D of this color bar.
     */
    private Rectangle2D getColorBar(int number) {
        int barWidth = this.width / this.colors;

        return new Rectangle2D.Double(barWidth * number, 0,
                                       barWidth, this.height);
    }

    /**
     * Get the Color object associated with
     * a certain palette number.
     *
     * @param number The color number.
     * @return The Color object associated with that number.
     */
    public Color getPaletteColor(int number) {
        Color c = Color.PINK;

        if (number == 0) {
            c = Color.RED;
        } else if (number == 1) {
            c = Color.BLUE;
        } else if (number == 2) {
            c = Color.GREEN;
        } else if (number == 3) {
            c = Color.MAGENTA;
        } else if (number == 4) {
            c = Color.YELLOW;
        } else if (number == 5) {
            c = Color.CYAN;
        } else if (number == 6) {
            c = Color.GRAY;
        } else if (number == 7) {
            c = Color.DARK_GRAY;
        } else if (number == 8) {
            c = Color.LIGHT_GRAY;
        } else if (number == 9) {
            c = Color.ORANGE;
        }

        return c;
    }

    /**
     * Return the selected color.
     *
     * @return The selected color on the ColorBar.
     */
    public int getColor() {
        return this.color;
    }

    /**
     * Set the selected color on the ColorBar.
     *
     * @param color The selected color.
     */
    public void setColor(int color) {
        this.color = color;
    }

    /**
     * Get the number of colors on the ColorBar.
     *
     * @return The number of colors.
     */
    public int getColors() {
        return this.colors;
    }

    /**
     * Set the number of colors on the ColorBar.
     *
     * @param colors The number of colors on the ColorBar.
     */
    public void setColors(int colors) {
        this.colors = colors;
    }

    /**
     * Get the width of the ColorBar.
     *
     * @return The width of the ColorBar.
     */
    public int getWidth() {
        return this.width;
    }

    /**
     * Set the width of the ColorBar.
     *
     * @param width The new width of the ColorBar.
     */
    public void setWidth(int width) {
        this.width = width;
    }
    
    /**
     * Get the height of the ColorBar.
     *
     * @return The height of the ColorBar.
     */
    public int getHeight() {
        return this.height;
    }

    /**
     * Set the width of the ColorBar.
     *
     * @param width The new width of the ColorBar.
     */
    public void setHeight(int height) {
        this.height = height;
    }

    /**
     * Paint the color bar using the pen.
     *
     * @param pen The Graphics context to use.
     */
    public void draw(Graphics2D pen) {
        int barWidth = this.width / this.colors;
        int loop = 0;

        // paint the color Rectangles
        while (loop < this.colors) {
            pen.setPaint(getPaletteColor(loop));
            pen.fill(getColorBar(loop));

            loop++;
        }

        // set the rectangle selection color
        pen.setPaint(Color.WHITE);

        // draw the selection Rectangle over the selected ColorBar.
        pen.draw(new Rectangle2D.Double((this.color * barWidth) + 5, 5,
                                        (barWidth - 10), (this.height - 10)));
    }
}

/**
 * @author John David Ratliff [jdratlif@indiana.edu]
 * @version 1.0.0
 *
 * Class representing a Dot to be used as Confetti.
 */
class Dot {
    private int radius;
    private Point p;

    /**
     * Creates a new Dot to be drawn centered at p with radius radius.
     *
     * @param p The center of the new Dot.
     * @param radius The radius of the circular dot.
     */
    Dot(Point p, int radius) {
        this.p = p;
        this.radius = radius;

        // get the correct bounding box offsets
        this.p.translate(-radius, -radius);
    }

    /**
     * Draws the Dot using the Graphics pen provided.
     *
     * @param pen The Graphics pen to draw with.
     */
    public void draw(Graphics2D pen) {
        pen.fill(new Ellipse2D.Double(p.x,p.y,radius * 2,radius * 2));
    }
}
